@@ -1,7 +1,7 @@
 import logging
 from datetime import timedelta
 from sqlalchemy import and_
-from sqlalchemy.exc import IntegrityError
+from sqlalchemy.exc import IntegrityError, InvalidRequestError
 from sqlalchemy.orm.exc import ObjectDeletedError
 
 from vwsfriend.model.charge import Charge
@@ -19,7 +19,7 @@ LOG = logging.getLogger("VWsFriend")
 class ChargeAgent():
     def __init__(self, session, vehicle, privacy):
         self.session = session
-        self.vehicle = vehicle
+        self.vehicle = session.merge(vehicle)
         self.privacy = privacy
         if Privacy.NO_LOCATIONS in self.privacy:
             LOG.info(f'Privacy option \'no-locations\' is set. Vehicle {self.vehicle.vin} will not record charging locations')
@@ -47,8 +47,14 @@ class ChargeAgent():
 
                 # If the vehicle is charging check if you can catch up an open charging session:
                 if self.vehicle.weConnectVehicle.domains['charging']['chargingStatus'].chargingState.enabled \
-                        and self.vehicle.weConnectVehicle.domains['charging']['chargingStatus'].chargingState.value == ChargingStatus.ChargingState.CHARGING:
-                    chargingSession = session.query(ChargingSession).filter(ChargingSession.vehicle == vehicle).order_by(ChargingSession.started.desc()).first()
+                        and self.vehicle.weConnectVehicle.domains['charging']['chargingStatus'].chargingState.value in \
+                        (ChargingStatus.ChargingState.CHARGING,
+                         ChargingStatus.ChargingState.CONSERVATION,
+                         ChargingStatus.ChargingState.CHARGE_PURPOSE_REACHED_NOT_CONSERVATION_CHARGING,
+                         ChargingStatus.ChargingState.CHARGE_PURPOSE_REACHED_CONSERVATION,
+                         ChargingStatus.ChargingState.DISCHARGING):
+                    chargingSession = session.query(ChargingSession).filter(and_(ChargingSession.vehicle == vehicle, ChargingSession.started.isnot(None))
+                                                                            ).order_by(ChargingSession.started.desc()).first()
                     if chargingSession is not None and not chargingSession.isClosed():
                         self.chargingSession = chargingSession
                         LOG.info('Vehicle is charging and an open charging session entry was found in the database. This session will be continued.')
@@ -64,6 +70,17 @@ class ChargeAgent():
                                                                                                           AddressableLeaf.ObserverEvent.VALUE_CHANGED,
                                                                                                           onUpdateComplete=True)
 
+                # If the vehicle is still connected check if you can catch up an open charging session:
+                if self.chargingSession is None and self.vehicle.weConnectVehicle.domains['charging']['plugStatus'].plugConnectionState.value \
+                        == PlugStatus.PlugConnectionState.CONNECTED:
+                    chargingSession = session.query(ChargingSession).filter(and_(ChargingSession.vehicle == vehicle, ChargingSession.connected.isnot(None))
+                                                                            ).order_by(ChargingSession.connected.desc()).first()
+                    if chargingSession is not None and not chargingSession.wasDisconnected():
+                        self.chargingSession = chargingSession
+                        LOG.info('Vehicle is still connected and an open charging session entry was found in the database. This session will be continued.')
+                    else:
+                        LOG.warning('Vehicle still connected but no open charging session entry was found in the database. This session cannot be recorded.')
+
     def __onChargingStatusCarCapturedTimestampChange(self, element, flags):  # noqa: C901
         if element is not None and element.value is not None:
             chargeStatus = self.vehicle.weConnectVehicle.domains['charging']['chargingStatus']
@@ -90,6 +107,10 @@ class ChargeAgent():
                     LOG.warning('Last charge entry was deleted')
                     self.charge = self.session.query(Charge).filter(and_(Charge.vehicle == self.vehicle, Charge.carCapturedTimestamp.isnot(None))) \
                         .order_by(Charge.carCapturedTimestamp.desc()).first()
+                except InvalidRequestError:
+                    LOG.warning('Last charge entry was not persisted')
+                    self.charge = self.session.query(Charge).filter(and_(Charge.vehicle == self.vehicle, Charge.carCapturedTimestamp.isnot(None))) \
+                        .order_by(Charge.carCapturedTimestamp.desc()).first()
 
             if self.charge is None or (self.charge.carCapturedTimestamp != chargeStatus.carCapturedTimestamp.value and (
                     self.charge.remainingChargingTimeToComplete_min != current_remainingChargingTimeToComplete_min
@@ -100,12 +121,12 @@ class ChargeAgent():
 
                 self.charge = Charge(self.vehicle, chargeStatus.carCapturedTimestamp.value, current_remainingChargingTimeToComplete_min, current_chargingState,
                                      current_chargeMode, current_chargePower_kW, current_chargeRate_kmph)
-                try:
-                    with self.session.begin_nested():
+                with self.session.begin_nested():
+                    try:
                         self.session.add(self.charge)
-                    self.session.commit()
-                except IntegrityError as err:
-                    LOG.warning('Could not add charge entry to the database, this is usually due to an error in the WeConnect API (%s)', err)
+                    except IntegrityError as err:
+                        LOG.warning('Could not add charge entry to the database, this is usually due to an error in the WeConnect API (%s)', err)
+                self.session.commit()
 
     def __onChargingStateChange(self, element, flags):  # noqa: C901
         chargeStatus = self.vehicle.weConnectVehicle.domains['charging']['chargingStatus']
@@ -122,75 +143,82 @@ class ChargeAgent():
                     self.chargingSession = chargingSession
                 else:
                     self.chargingSession = None
+            except InvalidRequestError:
+                LOG.warning('Last charging session was not persisted')
+                chargingSession = self.session.query(ChargingSession).filter(ChargingSession.vehicle == self.vehicle) \
+                    .order_by(ChargingSession.started.desc()).first()
+                if chargingSession is not None and not chargingSession.isClosed():
+                    self.chargingSession = chargingSession
+                else:
+                    self.chargingSession = None
 
         if element.value in [ChargingStatus.ChargingState.CHARGING, ChargingStatus.ChargingState.CHARGE_PURPOSE_REACHED_CONSERVATION,
                              ChargingStatus.ChargingState.CONSERVATION]:
-            if self.chargingSession is None or self.chargingSession.isClosed():
-                # In case this was an interrupted charging session (interrupt no longer than 24hours), continue by erasing end time
-                if self.chargingSession is not None and not self.chargingSession.wasDisconnected() \
-                        and (self.chargingSession.ended is None or chargeStatus.carCapturedTimestamp.value is None
-                             or self.chargingSession.ended > (chargeStatus.carCapturedTimestamp.value - timedelta(hours=24))):
-                    self.chargingSession.ended = None
-                    self.chargingSession.endSOC_pct = None
-                else:
-                    self.previousChargingSession = self.chargingSession
-                    self.chargingSession = ChargingSession(vehicle=self.vehicle)
-                    try:
-                        with self.session.begin_nested():
-                            self.session.add(self.chargingSession)
-                        self.session.commit()
-                    except IntegrityError:
-                        LOG.warning('Could not add charging session entry to the database, this is usually due to an error in the WeConnect API')
-            if not self.chargingSession.wasStarted():
-                self.chargingSession.started = chargeStatus.carCapturedTimestamp.value
-            # also write start SoC
-            if self.chargingSession is not None and self.chargingSession.startSOC_pct is None \
-                    and self.vehicle.weConnectVehicle.statusExists('charging', 'batteryStatus'):
-                batteryStatus = self.vehicle.weConnectVehicle.domains['charging']['batteryStatus']
-                if batteryStatus.enabled and batteryStatus.currentSOC_pct.enabled:
-                    self.chargingSession.startSOC_pct = batteryStatus.currentSOC_pct.value
-            # also write start charge type if available and not already set
-            if self.chargingSession is not None and self.chargingSession.acdc is None \
-                    and chargeStatus.chargeType.enabled:
-                if chargeStatus.chargeType.value == ChargingStatus.ChargeType.AC:
-                    self.chargingSession.acdc = ACDC.AC
-                elif chargeStatus.chargeType.value == ChargingStatus.ChargeType.DC:
-                    self.chargingSession.acdc = ACDC.DC
-
-            # also write position if available
-            self.updatePosition()
-
-            # also write milage if available
-            self.updateMileage()
-
-            self.session.commit()
-        elif element.value in [ChargingStatus.ChargingState.OFF, ChargingStatus.ChargingState.READY_FOR_CHARGING,
-                               ChargingStatus.ChargingState.NOT_READY_FOR_CHARGING,
-                               ChargingStatus.ChargingState.CHARGE_PURPOSE_REACHED_NOT_CONSERVATION_CHARGING,
-                               ChargingStatus.ChargingState.ERROR]:
-            if self.chargingSession is not None and self.chargingSession.isChargingState():
-                self.chargingSession.ended = chargeStatus.carCapturedTimestamp.value
-
-                # also write start charge type if not already set before
-                if self.chargingSession.maximumChargePower_kW is not None:
-                    if self.chargingSession.maximumChargePower_kW > 11:
-                        self.chargingSession.acdc = ACDC.DC
+            with self.session.begin_nested():
+                if self.chargingSession is None or self.chargingSession.isClosed():
+                    # In case this was an interrupted charging session (interrupt no longer than 24hours), continue by erasing end time
+                    if self.chargingSession is not None and not self.chargingSession.wasDisconnected() \
+                            and (self.chargingSession.ended is None or chargeStatus.carCapturedTimestamp.value is None
+                                 or self.chargingSession.ended > (chargeStatus.carCapturedTimestamp.value - timedelta(hours=24))):
+                        self.chargingSession.ended = None
+                        self.chargingSession.endSOC_pct = None
                     else:
-                        self.chargingSession.acdc = ACDC.AC
-                # also write end SoC
-                if self.chargingSession is not None and self.chargingSession.endSOC_pct is None \
+                        self.previousChargingSession = self.chargingSession
+                        self.chargingSession = ChargingSession(vehicle=self.vehicle)
+                        try:
+                            self.session.add(self.chargingSession)
+                        except IntegrityError:
+                            LOG.warning('Could not add charging session entry to the database, this is usually due to an error in the WeConnect API')
+                if not self.chargingSession.wasStarted():
+                    self.chargingSession.started = chargeStatus.carCapturedTimestamp.value
+                # also write start SoC
+                if self.chargingSession is not None and self.chargingSession.startSOC_pct is None \
                         and self.vehicle.weConnectVehicle.statusExists('charging', 'batteryStatus'):
                     batteryStatus = self.vehicle.weConnectVehicle.domains['charging']['batteryStatus']
                     if batteryStatus.enabled and batteryStatus.currentSOC_pct.enabled:
-                        self.chargingSession.endSOC_pct = batteryStatus.currentSOC_pct.value
+                        self.chargingSession.startSOC_pct = batteryStatus.currentSOC_pct.value
+                # also write start charge type if available and not already set
+                if self.chargingSession is not None and self.chargingSession.acdc is None \
+                        and chargeStatus.chargeType.enabled:
+                    if chargeStatus.chargeType.value == ChargingStatus.ChargeType.AC:
+                        self.chargingSession.acdc = ACDC.AC
+                    elif chargeStatus.chargeType.value == ChargingStatus.ChargeType.DC:
+                        self.chargingSession.acdc = ACDC.DC
 
                 # also write position if available
                 self.updatePosition()
 
                 # also write milage if available
                 self.updateMileage()
+            self.session.commit()
 
-                self.session.commit()
+        elif element.value in [ChargingStatus.ChargingState.OFF, ChargingStatus.ChargingState.READY_FOR_CHARGING,
+                               ChargingStatus.ChargingState.NOT_READY_FOR_CHARGING,
+                               ChargingStatus.ChargingState.CHARGE_PURPOSE_REACHED_NOT_CONSERVATION_CHARGING,
+                               ChargingStatus.ChargingState.ERROR]:
+            with self.session.begin_nested():
+                if self.chargingSession is not None and self.chargingSession.isChargingState():
+                    self.chargingSession.ended = chargeStatus.carCapturedTimestamp.value
+
+                    # also write start charge type if not already set before
+                    if self.chargingSession.maximumChargePower_kW is not None:
+                        if self.chargingSession.maximumChargePower_kW > 11:
+                            self.chargingSession.acdc = ACDC.DC
+                        else:
+                            self.chargingSession.acdc = ACDC.AC
+                    # also write end SoC
+                    if self.chargingSession is not None and self.chargingSession.endSOC_pct is None \
+                            and self.vehicle.weConnectVehicle.statusExists('charging', 'batteryStatus'):
+                        batteryStatus = self.vehicle.weConnectVehicle.domains['charging']['batteryStatus']
+                        if batteryStatus.enabled and batteryStatus.currentSOC_pct.enabled:
+                            self.chargingSession.endSOC_pct = batteryStatus.currentSOC_pct.value
+
+                    # also write position if available
+                    self.updatePosition()
+
+                    # also write milage if available
+                    self.updateMileage()
+            self.session.commit()
 
     def __onPlugConnectionStateChange(self, element, flags):  # noqa: C901
         plugStatus = self.vehicle.weConnectVehicle.domains['charging']['plugStatus']
@@ -207,33 +235,40 @@ class ChargeAgent():
                     self.chargingSession = chargingSession
                 else:
                     self.chargingSession = None
+            except InvalidRequestError:
+                LOG.warning('Last charging session was not persisted')
+                chargingSession = self.session.query(ChargingSession).filter(ChargingSession.vehicle == self.vehicle) \
+                    .order_by(ChargingSession.started.desc()).first()
+                if chargingSession is not None and not chargingSession.isClosed():
+                    self.chargingSession = chargingSession
+                else:
+                    self.chargingSession = None
 
         if element.value == PlugStatus.PlugConnectionState.CONNECTED:
-            if self.chargingSession is None or self.chargingSession.isClosed():
-                self.previousChargingSession = self.chargingSession
-                self.chargingSession = ChargingSession(vehicle=self.vehicle)
-                try:
-                    with self.session.begin_nested():
+            with self.session.begin_nested():
+                if self.chargingSession is None or self.chargingSession.isClosed():
+                    self.previousChargingSession = self.chargingSession
+                    self.chargingSession = ChargingSession(vehicle=self.vehicle)
+                    try:
                         self.session.add(self.chargingSession)
-                    self.session.commit()
-                except IntegrityError as err:
-                    LOG.warning('Could not add charging session entry to the database, this is usually due to an error in the WeConnect API (%s)', err)
-            if self.chargingSession.connected is None:
-                self.chargingSession.connected = plugStatus.carCapturedTimestamp.value
-            # also write position if available
-            self.updatePosition()
-            # also write milage if available
-            self.updateMileage()
-
+                    except IntegrityError as err:
+                        LOG.warning('Could not add charging session entry to the database, this is usually due to an error in the WeConnect API (%s)', err)
+                if self.chargingSession.connected is None:
+                    self.chargingSession.connected = plugStatus.carCapturedTimestamp.value
+                # also write position if available
+                self.updatePosition()
+                # also write milage if available
+                self.updateMileage()
             self.session.commit()
-        elif element.value == PlugStatus.PlugConnectionState.DISCONNECTED:
-            if self.chargingSession is not None and self.chargingSession.isConnectedState():
-                self.chargingSession.disconnected = plugStatus.carCapturedTimestamp.value
-            # also write position if available
-            self.updatePosition()
-            # also write milage if available
-            self.updateMileage()
 
+        elif element.value == PlugStatus.PlugConnectionState.DISCONNECTED:
+            with self.session.begin_nested():
+                if self.chargingSession is not None and self.chargingSession.isConnectedState():
+                    self.chargingSession.disconnected = plugStatus.carCapturedTimestamp.value
+                # also write position if available
+                self.updatePosition()
+                # also write milage if available
+                self.updateMileage()
             self.session.commit()
 
     def __onPlugLockStateChange(self, element, flags):  # noqa: C901
@@ -251,36 +286,43 @@ class ChargeAgent():
                     self.chargingSession = chargingSession
                 else:
                     self.chargingSession = None
+            except InvalidRequestError:
+                LOG.warning('Last charging session was not persisted')
+                chargingSession = self.session.query(ChargingSession).filter(ChargingSession.vehicle == self.vehicle) \
+                    .order_by(ChargingSession.started.desc()).first()
+                if chargingSession is not None and not chargingSession.isClosed():
+                    self.chargingSession = chargingSession
+                else:
+                    self.chargingSession = None
 
         if element.value == PlugStatus.PlugLockState.LOCKED:
-            if self.chargingSession is None or self.chargingSession.isClosed():
-                # In case this was an interrupted charging session (interrupt no longer than 24hours), continue by erasing end time
-                if self.chargingSession is not None and not self.chargingSession.wasDisconnected() \
-                        and (self.chargingSession.unlocked is None or plugStatus.carCapturedTimestamp.value is None
-                             or self.chargingSession.unlocked > (plugStatus.carCapturedTimestamp.value - timedelta(hours=24))):
-                    self.chargingSession.unlocked = None
-                else:
-                    self.previousChargingSession = self.chargingSession
-                    self.chargingSession = ChargingSession(vehicle=self.vehicle)
-                    with self.session.begin_nested():
+            with self.session.begin_nested():
+                if self.chargingSession is None or self.chargingSession.isClosed():
+                    # In case this was an interrupted charging session (interrupt no longer than 24hours), continue by erasing end time
+                    if self.chargingSession is not None and not self.chargingSession.wasDisconnected() \
+                            and (self.chargingSession.unlocked is None or plugStatus.carCapturedTimestamp.value is None
+                                 or self.chargingSession.unlocked > (plugStatus.carCapturedTimestamp.value - timedelta(hours=24))):
+                        self.chargingSession.unlocked = None
+                    else:
+                        self.previousChargingSession = self.chargingSession
+                        self.chargingSession = ChargingSession(vehicle=self.vehicle)
                         self.session.add(self.chargingSession)
-                    self.session.commit()
-            if self.chargingSession.locked is None:
-                self.chargingSession.locked = plugStatus.carCapturedTimestamp.value
-            # also write position if available
-            self.updatePosition()
-            # also write milage if available
-            self.updateMileage()
-
+                if self.chargingSession.locked is None:
+                    self.chargingSession.locked = plugStatus.carCapturedTimestamp.value
+                # also write position if available
+                self.updatePosition()
+                # also write milage if available
+                self.updateMileage()
             self.session.commit()
-        elif element.value == PlugStatus.PlugLockState.UNLOCKED:
-            if self.chargingSession is not None and self.chargingSession.isLockedState():
-                self.chargingSession.unlocked = plugStatus.carCapturedTimestamp.value
-            # also write position if available
-            self.updatePosition()
-            # also write milage if available
-            self.updateMileage()
 
+        elif element.value == PlugStatus.PlugLockState.UNLOCKED:
+            with self.session.begin_nested():
+                if self.chargingSession is not None and self.chargingSession.isLockedState():
+                    self.chargingSession.unlocked = plugStatus.carCapturedTimestamp.value
+                # also write position if available
+                self.updatePosition()
+                # also write milage if available
+                self.updateMileage()
             self.session.commit()
 
     def __onChargePowerChange(self, element, flags):
@@ -296,11 +338,19 @@ class ChargeAgent():
                     self.chargingSession = chargingSession
                 else:
                     self.chargingSession = None
+            except InvalidRequestError:
+                LOG.warning('Last charging session was not persisted')
+                chargingSession = self.session.query(ChargingSession).filter(ChargingSession.vehicle == self.vehicle) \
+                    .order_by(ChargingSession.started.desc()).first()
+                if chargingSession is not None and not chargingSession.isClosed():
+                    self.chargingSession = chargingSession
+                else:
+                    self.chargingSession = None
 
         if self.chargingSession is not None and self.chargingSession.isChargingState()\
                 and (self.chargingSession.maximumChargePower_kW is None or element.value > self.chargingSession.maximumChargePower_kW):
-            self.chargingSession.maximumChargePower_kW = element.value
-
+            with self.session.begin_nested():
+                self.chargingSession.maximumChargePower_kW = element.value
             self.session.commit()
 
     def updatePosition(self):
@@ -326,4 +376,4 @@ class ChargeAgent():
                 self.chargingSession.mileage_km = odometerMeasurement.odometer.value
 
     def commit(self):
-        pass
+        self.session.commit()
