@@ -1,4 +1,6 @@
+import logging
 import requests
+from requests.adapters import HTTPAdapter, Retry
 from haversine import haversine, Unit, inverse_haversine, Direction
 from sqlalchemy import and_
 
@@ -8,6 +10,8 @@ from vwsfriend.model.charger import Charger, Operator
 
 from weconnect.errors import RetrievalError
 
+LOG = logging.getLogger("VWsFriend")
+
 
 def locationFromLatLonWithGeofence(session, latitude, longitude):
     if latitude is None or longitude is None:
@@ -32,10 +36,16 @@ def locationFromLatLon(session, latitude, longitude):
         'User-Agent': 'VWsFriend'
     }
 
-    response = requests.get('https://nominatim.openstreetmap.org/reverse', params=query, headers=headers)
-    if response.status_code == requests.codes['ok']:
-        location = Location(jsonDict=response.json())
-        return session.merge(location)
+    openstreetmapSession = requests.Session()
+    retries = Retry(total=3, backoff_factor=0.1, status_forcelist=[500], raise_on_status=False)
+    openstreetmapSession.mount('https://', HTTPAdapter(max_retries=retries))
+    try:
+        response = openstreetmapSession.get('https://nominatim.openstreetmap.org/reverse', params=query, headers=headers)
+        if response.status_code == requests.codes['ok']:
+            location = Location(jsonDict=response.json())
+            return session.merge(location)
+    except requests.exceptions.RetryError as retryError:
+        LOG.error('Could not retrieve location: %s', retryError)
     return None
 
 
@@ -53,17 +63,23 @@ def amenityFromLatLon(session, latitude, longitude, radius, amenity, withFallbac
     headers = {
         'User-Agent': 'VWsFriend'
     }
-    response = requests.get('https://nominatim.openstreetmap.org/search', params=query, headers=headers)
-    if response.status_code == requests.codes['ok']:
-        places = response.json()
-        placesDistance = [(haversine((latitude, longitude), (float(place['lat']), float(place['lon'])), unit=Unit.METERS), place) for place in places]
-        placesDistance = sorted(placesDistance, key=lambda geofence: geofence[0])
-        for distance, place in placesDistance:
-            if distance < radius:
-                location = Location(jsonDict=place)
-                return session.merge(location)
-    if withFallback:
-        return locationFromLatLon(session, latitude, longitude)
+    openstreetmapSession = requests.Session()
+    retries = Retry(total=3, backoff_factor=0.1, status_forcelist=[500], raise_on_status=False)
+    openstreetmapSession.mount('https://', HTTPAdapter(max_retries=retries))
+    try:
+        response = openstreetmapSession.get('https://nominatim.openstreetmap.org/search', params=query, headers=headers)
+        if response.status_code == requests.codes['ok']:
+            places = response.json()
+            placesDistance = [(haversine((latitude, longitude), (float(place['lat']), float(place['lon'])), unit=Unit.METERS), place) for place in places]
+            placesDistance = sorted(placesDistance, key=lambda geofence: geofence[0])
+            for distance, place in placesDistance:
+                if distance < radius:
+                    location = Location(jsonDict=place)
+                    return session.merge(location)
+        if withFallback:
+            return locationFromLatLon(session, latitude, longitude)
+    except requests.exceptions.RetryError as retryError:
+        LOG.error('Could not retrieve location: %s', retryError)
     return None
 
 
@@ -125,7 +141,8 @@ def addCharger(session, weConnectCharger):
     if weConnectCharger.chargingSpots.enabled:
         charger.num_spots = len(weConnectCharger.chargingSpots)
 
-    charger.operator = Operator(id=weConnectCharger.operator.id.value, name=weConnectCharger.operator.name.value,
-                                phone=weConnectCharger.operator.phoneNumber.value)
+    if weConnectCharger.operator is not None and weConnectCharger.operator.enabled:
+        charger.operator = Operator(id=weConnectCharger.operator.id.value, name=weConnectCharger.operator.name.value,
+                                    phone=weConnectCharger.operator.phoneNumber.value)
 
     return session.merge(charger)
