@@ -14,7 +14,7 @@ import threading
 from pyhap.accessory_driver import AccessoryDriver
 
 from weconnect import weconnect
-from weconnect.errors import APICompatibilityError, AuthentificationError
+from weconnect.errors import APICompatibilityError, AuthentificationError, TemporaryAuthentificationError
 from weconnect.util import DuplicateFilter
 from weconnect.__version import __version__ as __weconnect_version__
 
@@ -34,6 +34,7 @@ try:
 
     from weconnect_mqtt. weconnect_mqtt_base import PictureFormat, WeConnectMQTTClient  # type: ignore
     from weconnect_mqtt.__version import __version__ as __weconnect_mqtt_version__
+    import paho.mqtt.client
     SUPPORT_MQTT = True
 except ImportError:
     pass
@@ -137,6 +138,10 @@ def main():  # noqa: C901 pylint: disable=too-many-branches, too-many-statements
     loggingMailGroup.add_argument('--logging-mail-notls', dest='loggingMailnotls', help='Mail do not use TLS', required=False, action='store_true')
     loggingMailGroup.add_argument('--logging-mail-testmail', dest='loggingMailTestmail', help='Try to send Testmail at startup', required=False,
                                   action='store_true')
+    loggingMailGroup.add_argument('--logging-mail-filter-duplicates', dest='loggingMailFilterDuplicates', help='Filter duplicated messages out', required=False,
+                                  action='store_true')
+    loggingMailGroup.add_argument('--logging-mail-filter-reset', dest='loggingMailFilterReset', help='Reset duplicate filter after number of seconds', type=int,
+                                  required=False, default=0)
 
     if SUPPORT_MQTT:
         mqttGroup = parser.add_argument_group('MQTT', description='MQTT support in VWsFriend is EXPERIMENTAL,'
@@ -150,6 +155,8 @@ def main():  # noqa: C901 pylint: disable=too-many-branches, too-many-statements
         mqttGroup.add_argument('-k', '--mqttkeepalive', required=False, type=int, default=60, help='Time between keep-alive messages')
         mqttGroup.add_argument('-mu', '--mqtt-username', type=str, dest='mqttusername', help='Username for MQTT broker', required=False)
         mqttGroup.add_argument('-mp', '--mqtt-password', type=str, dest='mqttpassword', help='Password for MQTT broker', required=False)
+        mqttGroup.add_argument('-mv', '--mqtt-version', type=str, dest='mqttversion', help='MQTT protocol version used', required=False,
+                               choices=['3.1', '3.1.1', '5'], default='3.1.1')
         mqttGroup.add_argument('--transport', required=False, default='tcp', choices=["tcp", 'websockets'],
                                help='EXPERIMENTAL support for websockets transport')
         mqttGroup.add_argument('-s', '--use-tls', action='store_true', help='EXPERIMENTAL')
@@ -214,6 +221,8 @@ def main():  # noqa: C901 pylint: disable=too-many-branches, too-many-statements
                                                        secure=secure)
             smtpHandler.setLevel(logging.INFO)
             smtpHandler.setFormatter(logging.Formatter("%(asctime)s %(levelname)-5s %(message)s"))
+            if args.loggingMailFilterDuplicates:
+                smtpHandler.addFilter(DuplicateFilter(filterResetSeconds=args.loggingMailFilterReset))
             LOG.addHandler(smtpHandler)
             if args.loggingMailTestmail:
                 if SUPPORT_MQTT:
@@ -316,7 +325,7 @@ def main():  # noqa: C901 pylint: disable=too-many-branches, too-many-statements
     weConnect = None
     try:  # pylint: disable=too-many-nested-blocks
         weConnect = weconnect.WeConnect(username=weConnectUsername, password=weConnectPassword, spin=weConnectSpin, tokenfile=tokenfile,
-                                        updateAfterLogin=False, loginOnInit=(args.demo is None), maxAgePictures=86400, forceReloginAfter=21600)
+                                        updateAfterLogin=False, loginOnInit=(args.demo is None), maxAgePictures=86400, forceReloginAfter=21600, numRetries=5)
 
         connector = AgentConnector(weConnect=weConnect, dbUrl=args.dbUrl, interval=args.interval, withDB=args.withDatabase, withABRP=args.withABRP,
                                    configDir=args.configDir, privacy=args.privacy)
@@ -403,11 +412,19 @@ def main():  # noqa: C901 pylint: disable=too-many-branches, too-many-statements
                 weConnect.longitude = longitude
                 weConnect.searchRadius = args.chargingLocationRadius
 
-            mqttCLient = WeConnectMQTTClient(clientId=args.mqttclientid, transport=args.transport, interval=args.interval,
-                                             prefix=args.prefix, ignore=args.ignore, updatePictures=args.pictures, listNewTopics=args.listTopics,
-                                             republishOnUpdate=args.republishOnUpdate, pictureFormat=args.pictureFormat, topicFilterRegex=topicFilterRegex,
-                                             convertTimezone=convertTimezone, timeFormat=args.timeFormat, withRawJsonTopic=args.withRawJsonTopic,
-                                             passive=True)
+            if args.mqttversion == '3.1':
+                mqttVersion = paho.mqtt.client.MQTTv31
+            elif args.mqttversion == '5':
+                mqttVersion = paho.mqtt.client.MQTTv5
+            else:
+                mqttVersion = paho.mqtt.client.MQTTv311
+
+            mqttCLient = WeConnectMQTTClient(clientId=args.mqttclientid, protocol=mqttVersion, transport=args.transport, interval=args.interval,
+                                             prefix=args.prefix, ignore=args.ignore, updateCapabilities=True, updatePictures=True,
+                                             listNewTopics=args.listTopics, republishOnUpdate=args.republishOnUpdate, pictureFormat=args.pictureFormat,
+                                             topicFilterRegex=topicFilterRegex, convertTimezone=convertTimezone, timeFormat=args.timeFormat,
+                                             withRawJsonTopic=args.withRawJsonTopic, passive=False, updateOnConnect=False)
+
             mqttCLient.enable_logger()
 
             if usetls:
@@ -449,6 +466,7 @@ def main():  # noqa: C901 pylint: disable=too-many-branches, too-many-statements
                 # blocking run
                 mqttCLient.loop_forever(retry_first_connection=True)
                 mqttCLient.disconnect()
+                LOG.error('MQTT Connection failed')
 
             mqttThread = threading.Thread(target=mqttWorker)
             mqttThread.start()
@@ -486,24 +504,44 @@ def main():  # noqa: C901 pylint: disable=too-many-branches, too-many-statements
             LOG.info('Demo completed')
         else:
             starttime = time.time()
+            subsequentErrors = 0
             permanentErrors = 0
             sleeptime = args.interval
             while True:
                 try:
-                    LOG.info('Updating data from WeConnect')
-                    weConnect.update(updateCapabilities=True, updatePictures=True, force=True)
+                    if SUPPORT_MQTT and args.mqttbroker:
+                        mqttCLient.updateWeConnect(reraise=True)
+                    else:
+                        LOG.info('Updating data from WeConnect')
+                        weConnect.update(updateCapabilities=True, updatePictures=True, force=True)
                     connector.commit()
                     if args.withHomekit and not weConnectBridgeInitialized:
                         weConnectBridgeInitialized = True
                         bridge.update()
                     sleeptime = args.interval - ((time.time() - starttime) % args.interval)
                     permanentErrors = 0
+                    subsequentErrors = 0
                 except weconnect.RetrievalError:
-                    LOG.error('Retrieval error during update. Will try again after configured interval of %ds', args.interval)
+                    if subsequentErrors > 0:
+                        LOG.error('Retrieval error during update. Will try again after configured interval of %ds', args.interval)
+                    else:
+                        LOG.warning('Retrieval error during update. Will try again after configured interval of %ds', args.interval)
+                    subsequentErrors += 1
+                except TemporaryAuthentificationError:
+                    if subsequentErrors > 0:
+                        LOG.error('Temporary error during reauthentification. Will try again after configured interval of %ds', args.interval)
+                    else:
+                        LOG.warning('Temporary error during reauthentification. Will try again after configured interval of %ds', args.interval)
+                    subsequentErrors += 1
                 except APICompatibilityError as e:
                     sleeptime = min((args.interval * pow(2, permanentErrors)), 86400)
-                    LOG.critical('There was a problem when communicating with WeConnect. If this problem persists please open a bug report: %s,'
-                                 ' will retry after %ds', e, sleeptime)
+                    if subsequentErrors > 0:
+                        LOG.critical('There was a problem when communicating with WeConnect. If this problem persists please open a bug report: %s,'
+                                     ' will retry after %ds', e, sleeptime)
+                    else:
+                        LOG.warning('There was a problem when communicating with WeConnect. If this problem persists please open a bug report: %s,'
+                                    ' will retry after %ds', e, sleeptime)
+                    subsequentErrors += 1
                     permanentErrors += 1
                 #  Execute exactly every interval but if it misses its deadline only after the next interval
                 time.sleep(sleeptime)
@@ -516,3 +554,6 @@ def main():  # noqa: C901 pylint: disable=too-many-branches, too-many-statements
     finally:
         if weConnect is not None:
             weConnect.disconnect()
+        if SUPPORT_MQTT and args.mqttbroker:
+            if mqttCLient is not None:
+                mqttCLient.disconnect()
